@@ -21,14 +21,14 @@ Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
    Geoff Voelker (voelker@cs.washington.edu) 7-29-94 */
 
 /* Adapted for XEmacs by David Hobley <david@spook-le0.cia.com.au> */
+/* Synced with FSF Emacs 19.34.6 by Marc Paquette <marcpa@cam.org> */
 
-#include "config.h"
+#include <config.h>
+#include "lisp.h"  /* for VALMASK */
 
 #include <stdlib.h>
-#include <stdio.h>
 
 #include "ntheap.h"
-#include "lisp.h"  /* for VALMASK */
 
 /* This gives us the page size and the size of the allocation unit on NT.  */
 SYSTEM_INFO sysinfo_cache;
@@ -42,6 +42,9 @@ int etext;
 int nt_major_version;
 int nt_minor_version;
 
+/* Distinguish between Windows NT and Windows 95.  */
+int os_subtype;
+
 /* Cache information describing the NT system for later use.  */
 void
 cache_system_info (void)
@@ -62,6 +65,11 @@ cache_system_info (void)
   nt_major_version = version.info.major;
   nt_minor_version = version.info.minor;
 
+  if (version.info.platform & 0x8000)
+    os_subtype = OS_WIN95;
+  else
+    os_subtype = OS_NT;
+
   /* Cache page size, allocation unit, processor type, etc.  */
   GetSystemInfo (&sysinfo_cache);
   syspage_mask = sysinfo_cache.dwPageSize - 1;
@@ -181,13 +189,19 @@ sbrk (unsigned long increment)
       if (!data_region_base)
 	return NULL;
 
+#ifndef USE_MINIMAL_TAGBITS
       /* Ensure that the addresses don't use the upper tag bits since
 	 the Lisp type goes there.  */
-      if (((unsigned long) data_region_base & ~VALMASK) != 0) 
+#ifdef USE_UNION_TYPE
+      if (((unsigned long) data_region_base & ~((1U << VALBITS) - 1)) != 0)
+#else
+      if (((unsigned long) data_region_base & ~VALMASK) != 0)
+#endif
 	{
 	  printf ("Error: The heap was allocated in upper memory.\n");
 	  exit (1);
 	}
+#endif
 
       data_region_end = data_region_base;
       real_data_region_end = data_region_end;
@@ -271,8 +285,10 @@ recreate_heap (char *executable_path)
      any funny interactions between file I/O and file mapping.  */
   read_in_bss (executable_path);
   map_in_heap (executable_path);
-}
 
+  /* Update system version information to match current system.  */
+  cache_system_info ();
+}
 #endif /* CANNOT_DUMP */
 
 /* Round the heap up to the given alignment.  */
@@ -288,3 +304,26 @@ round_heap (unsigned long align)
   if (need_to_alloc) 
     sbrk (need_to_alloc);
 }
+
+#if (_MSC_VER >= 1000)
+
+/* MSVC 4.2 invokes these functions from mainCRTStartup to initialize
+   a heap via HeapCreate.  They are normally defined by the runtime,
+   but we override them here so that the unnecessary HeapCreate call
+   is not performed.  */
+
+int __cdecl
+_heap_init (void)
+{
+  /* Stepping through the assembly indicates that mainCRTStartup is
+     expecting a nonzero success return value.  */
+  return 1;
+}
+
+void __cdecl
+_heap_term (void)
+{
+  return;
+}
+
+#endif
