@@ -49,7 +49,7 @@ Boston, MA 02111-1307, USA.  */
 #include "device.h"
 #include "insdel.h"
 
-#ifdef __CYGWIN32__
+#if defined (__CYGWIN32__) && CYGWIN_VERSION_DLL_MAJOR < 21
 #define stricmp strcasecmp
 #define FONTENUMPROC FONTENUMEXPROC
 #define ntmTm ntmentm
