@@ -1,8 +1,8 @@
 #!/bin/sh
 emacs_major_version=21
 emacs_minor_version=1
-emacs_beta_version=7
-xemacs_codename="Biscayne"
+emacs_beta_version=8
+xemacs_codename="Bryce Canyon"
 infodock_major_version=4
 infodock_minor_version=0
 infodock_build_version=7
